﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using EssentialTools2.Models;
using Ninject;

namespace EssentialTools2.Infrastructure
{
    /// <summary>
    /// 依赖注入器类：MVC框架在需要一个类实例以便对一个传入的请求进行服务时，会调用GetService或GetServices方法
    /// 依赖解析器要做的工作便是创建这个实例：这一项需要调用Ninject的TryGet和GetAll方法来完成
    /// </summary>
    public class NinjectDependencyResolver:IDependencyResolver
    {
        private IKernel kernel;
        public NinjectDependencyResolver() {
            kernel = new StandardKernel();
            AddBindings();
        }        
        public object GetService(Type serviceType) {
            //TryGet方法所使用的类型参数告诉Ninject，用户感兴趣的是哪个接口，当没有合适的绑定时，会返回null，而不是抛出一个异常。
            return kernel.TryGet(serviceType);
        }
        public IEnumerable<object> GetServices(Type serviceType) {
            //GetAll方法支持对单一类型的多个绑定，当有多个不同的服务提供器时，可以使用它。
            return kernel.GetAll(serviceType);
        }
        private void AddBindings() {
            kernel.Bind<IValueCalculator>().To<LinqValueCalculator>();
            //kernel.Bind<IDiscountHelper>().To<DefaultDiscountHelper>().WithPropertyValue("DiscountSize",50m);
            kernel.Bind<IDiscountHelper>().To<DefaultDiscountHelper>().WithConstructorArgument("discountParam", 50m);
            kernel.Bind<IDiscountHelper>().To<FlexibleDiscountHelper>().WhenInjectedInto<LinqValueCalculator>();
        }
    }
}